@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
-from functools import lru_cache
 
 import abc
 import clu.abstract
@@ -17,7 +16,8 @@ from clu.config.keymapview import KeyMapKeysView, KeyMapItemsView, KeyMapValuesV
 from clu.config.keymapview import NamespaceWalkerKeysView, NamespaceWalkerItemsView
 from clu.config.keymapview import NamespaceWalkerValuesView
 
-from clu.predicates import (unwrap,
+from clu.naming import qualified_import, qualified_name, nameof
+from clu.predicates import (typeof,
                             isexpandable, iscontainer, isnotnone,
                             always, uncallable)
 
@@ -27,8 +27,6 @@ from clu.exporting import Exporter
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
-cache = lambda function: lru_cache(maxsize=128, typed=True)(function)
-
 # SUB-BASE AND ABSTRACT BASE CLASSES:
 
 @export
@@ -46,6 +44,17 @@ class FrozenKeyMapBase(collections.abc.Mapping,
     """
     __slots__ = pytuple('weakref')
     
+    @classmethod
+    @abstract
+    def from_dict(cls, instance_dict):
+        """ Used by `clu.config.codecs` to deserialize keymaps """
+        ...
+    
+    @abstract
+    def to_dict(self):
+        """ Used by `clu.config.codecs` to serialize the keymap """
+        ...
+    
     @abstract
     def namespaces(self):
         """ Iterate over all of the namespaces defined in the mapping. """
@@ -81,6 +90,12 @@ class FrozenKeyMapBase(collections.abc.Mapping,
     
     def __bool__(self):
         return len(self) > 0
+    
+    def __getstate__(self):
+        return self.to_dict()
+    
+    def __setstate__(self, instance_dict):
+        self.__init__(instance_dict)
 
 @export
 class KeyMapBase(FrozenKeyMapBase,
@@ -122,6 +137,11 @@ class FrozenKeyMap(FrozenKeyMapBase):
         supra. for further deets, my doggie
     """
     
+    @classmethod
+    def from_dict(cls, instance_dict):
+        """ Used by `clu.config.codecs` to deserialize keymaps """
+        return cls(instance_dict)
+    
     def get(self, key, *namespaces, default=NoDefault):
         """ Retrieve a (possibly namespaced) value for a given key.
             
@@ -174,7 +194,6 @@ class FrozenKeyMap(FrozenKeyMapBase):
     def __hash__(self):
         return hash(tuple(self.values()))
     
-    @cache
     def _get_namespace_foset(self):
         return FlatOrderedSet(get_ns(nskey) for nskey in sorted(self) if NAMESPACE_SEP in nskey)
     
@@ -192,6 +211,10 @@ class FrozenKeyMap(FrozenKeyMapBase):
     
     def namespace_count(self):
         return len(self._get_namespace_foset())
+    
+    def to_dict(self):
+        """ Used by `clu.config.codecs` to serialize the keymap """
+        return dict(self)
 
 @export
 class KeyMap(KeyMapBase, FrozenKeyMap):
@@ -229,6 +252,9 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
             if it doesn’t exist – raising a KeyError if no
             default value is given.
         """
+        # NoDefault never escapes to userland – the call to
+        # get(…) will raise the KeyError if NoDefault was passed
+        # (Q.v. “get(¬)” definition supra.)
         value = self.get(key, *namespaces, default=default)
         if value == default or value is default:
             return value
@@ -247,7 +273,7 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         iterator = iter(self)
         
         # Advance the iterator once, getting a single (possibly
-        # namespaced) key from the KeyMap, and raisting a KeyError
+        # namespaced) key from the KeyMap, and raising a KeyError
         # if there wasn’t anything found:
         try:
             nskey = next(iterator)
@@ -286,8 +312,6 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
                 self[nskey] = value
         if updates:
             self.update(dictish=updates)
-    
-    _get_namespace_foset = lambda self: unwrap(super()._get_namespace_foset)(self)
 
 # INTERIM ABSTRACT BASE: NamespaceWalker
 
@@ -356,7 +380,6 @@ class NamespaceWalker(FrozenKeyMap):
                 nss.add(concatenate_ns(*namespaces))
         yield from sorted(nss)
     
-    # @cache
     def _get_namespace_foset(self):
         return FlatOrderedSet(concatenate_ns(*ns) for *ns, _, _ in self.walk() if ns)
     
@@ -440,7 +463,11 @@ class FlatOrderedSet(collections.abc.Set,
         One can optionally specify, as a keyword-only argument, a unary boolean
         function “predicate” that will be used to filter out any of the items
         used to initialize the FlatOrderedSet for which the predicate returns
-        a Falsey value. 
+        a Falsey value.
+        
+        N.B. FlatOrderedSet serialized via the “clu.config.codecs” regime
+        have to employ properly defined, non-lambda predicates in order to
+        roundtrip properly through JSON or the pickle protocol.
     """
     __slots__ = ('things', 'predicate')
     
@@ -449,6 +476,12 @@ class FlatOrderedSet(collections.abc.Set,
         # Required by the “collections.abc.Set” API:
         return cls(*iterable)
     
+    @classmethod
+    def from_dict(cls, instance_dict):
+        """ Used by `clu.config.codecs` to deserialize FlatOrderedSets """
+        return cls(*instance_dict['things'],
+                    predicate=qualified_import(instance_dict['predicate']))
+    
     @classmethod
     def is_a(cls, instance):
         return isinstance(instance, (cls, FlatOrderedSet)) or \
@@ -497,7 +530,10 @@ class FlatOrderedSet(collections.abc.Set,
     def __getitem__(self, idx):
         if isinstance(idx, int):
             return self.things[idx]
-        return type(self)(*self.things[idx], predicate=self.predicate)
+        elif isinstance(idx, slice):
+            return type(self)(*self.things[idx], predicate=self.predicate)
+        badtype = nameof(typeof(idx))
+        raise TypeError(f"FlatOrderedSet indices must be integers or slices, not {badtype}")
     
     def __bool__(self):
         return len(self.things) > 0
@@ -519,6 +555,29 @@ class FlatOrderedSet(collections.abc.Set,
             return self.things != tuple(other)
         return NotImplemented
     
+    def __add__(self, operand):
+        if type(self).is_a(operand):
+            return type(self)(*self.things, *operand.things, predicate=self.predicate)
+        elif isinstance(operand, collections.abc.Sequence):
+            return type(self)(*self.things, *tuple(operand), predicate=self.predicate)
+        return NotImplemented
+    
+    def __radd__(self, operand):
+        if type(self).is_a(operand):
+            return type(self)(*operand.things, *self.things, predicate=self.predicate)
+        elif isinstance(operand, collections.abc.Sequence):
+            return type(self)(*tuple(operand), *self.things, predicate=self.predicate)
+        return NotImplemented
+    
+    def sort(self, key=None, reverse=False):
+        """ Return a new FlatOrderedSet with the contents sorted,
+            as per the usage of `builtins.sorted(…)`
+        """
+        return type(self)(sorted(self.things,
+                                 key=key,
+                                 reverse=reverse),
+                                 predicate=self.predicate)
+    
     def clone(self, deep=False, memo=None):
         # Q.v. https://stackoverflow.com/a/48550898/298171
         cls = type(self)
@@ -535,6 +594,12 @@ class FlatOrderedSet(collections.abc.Set,
         # which are superfluous as “inner_repr(…)” will
         # add its own parens by default
         return repr(self.things)[1:-1]
+    
+    def to_dict(self):
+        """ Used by `clu.config.codecs` to serialize FlatOrderedSets """
+        return { 'things'    : self.things,
+                 'predicate' : qualified_name(self.predicate) }
+    
 
 # CONCRETE CALLABLE SUBTYPES: functional_and, functional_set
 
