@@ -75,7 +75,7 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
         """ Articulate a flattened KeyMap instance out into one that is nested. """
         if cls is None:
             cls = FrozenNested
-        return cls(tree=articulate(self.dictionary))
+        return cls(tree=articulate(self.dictionary, walker=flatwalk))
     
     def __iter__(self):
         yield from self.dictionary
@@ -118,10 +118,14 @@ class Flat(FrozenFlat, KeyMap):
         del self.dictionary[nskey]
 
 @export
-def dictify(treeish):
-    """ Recursively convert a possibly nested mapping to standard dicts. """
+def dictify(treeish, cls=dict):
+    """ Recursively convert a possibly nested mapping to dicts.
+        
+        Standard Python dicts will be created by default; specify
+        a custom class with the “cls” argument to use your own.
+    """
     if ismapping(treeish):
-        return { key : dictify(value) for key, value in treeish.items() }
+        return cls({ key : dictify(value, cls=cls) for key, value in treeish.items() })
     return treeish
 
 @export
@@ -164,9 +168,18 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
             tree = getattr(tree, 'tree')
         elif hasattr(tree, 'nestify'):
             tree = getattr(tree.nestify(), 'tree')
-        self.tree = dictify(dict(tree or {}))
+        self.tree = dictify(dict(tree or {}), cls=dict)
         if updates:
-            self.tree.update(**updates)
+            for nskey, value in updates.items():
+                key, namespaces = unpack_ns(nskey)
+                d = self.tree
+                for namespace in namespaces:
+                    try:
+                        d = d[namespace]
+                    except KeyError:
+                        d[namespace] = {}
+                        d = d[namespace]
+                d[key] = value
     
     def walk(self):
         """ Iteratively walk the nested KeyMap’s tree of dicts. """
@@ -387,12 +400,12 @@ def test():
     @inline
     def test_frozennested_contains():
         """ FrozenNested contains namespaced key """
-        nested = FrozenNested(tree=nestedmaps(), **arbitrary())
+        nested = FrozenNested(tree=nestedmaps())
         
         for mappingpath in mapwalk(nested.tree):
             *namespaces, key, value = mappingpath
             nskey = pack_ns(key, *namespaces)
-            assert nskey in nested or nskey in arbitrary()
+            assert nskey in nested
     
     @inline
     def test_nested_contains():
