@@ -9,9 +9,11 @@ import clu.abstract
 import collections
 import collections.abc
 import contextlib
+import hashlib
 import importlib
 import inspect
 import itertools
+import shelve
 import sys, os
 import warnings
 import weakref
@@ -21,7 +23,7 @@ chain = itertools.chain
 iterchain = itertools.chain.from_iterable
 
 from clu.constants.consts import (λ, φ, # type: ignore
-                                  APPNAME, BASEPATH, BUILTINS,
+                                  APPNAME, BASEPATH, BUILTINS, ENCODING,
                                   EXPORTER_NAME, QUALIFIER, ROOT_PATH,
                                   NoDefault, pytuple)
 from clu.constants.exceptions import BadDotpathWarning, ExportError, ExportWarning
@@ -31,7 +33,6 @@ cache = lambda function: lru_cache(maxsize=128, typed=False)(function)
 
 # Not what you are looking for
 NotYourThing = object()
-# NotYourID = id(NotYourThing)
 
 def itermodule(module):
     """ Get an iterable of `(name, thing)` tuples for all things
@@ -182,6 +183,13 @@ def search_modules(thing, *modules):
                     return module, key
     return None, None
 
+@cache
+def stringhash(string):
+    """ Expediently hash a string to another string """
+    digester = hashlib.sha256()
+    digester.update(bytes(string, encoding=ENCODING))
+    return digester.hexdigest()
+
 def determine_name(thing, name=None, try_repr=False):
     """ Private module function to find a name for a thing. """
     # Shortcut everything if a name was explictly specified:
@@ -512,6 +520,44 @@ class ExporterBase(collections.abc.MutableMapping,
                 else:
                     self.__exports__.update(d)
     
+    def hash(self):
+        """ Return a stringified hash value corresponding to this
+            particular exporter instances’ dotpath.
+        """
+        return self.dotpath and stringhash(self.dotpath) or None
+    
+    def datafile(self):
+        """ Return a unique filename for this instance """
+        from clu.fs.appdirectories import AppDirs
+        from clu.constants.exceptions import FilesystemError
+        
+        # Initialize an AppDirs instance:
+        appdirs = AppDirs(appname=type(self).appname)
+        
+        # Ensure the user config directory exists:
+        try:
+            appdirs.user_config.makedirs()
+        except FilesystemError:
+            pass
+        
+        # Construct database filename:
+        selfhash = self.hash()
+        if not selfhash:
+            raise AttributeError("Could not hash the exporter instances’ dotpath")
+        return Path(appdirs.user_config / selfhash + ".pkl")
+    
+    @contextlib.contextmanager
+    def data(self, path=NoDefault):
+        """ Context manager for opening a shelving database, corresponding
+            to this particular exporter instances’ dotpath.
+        """
+        if path is NoDefault:
+            path = self.datafile()
+        
+        # Yield out a proper shelving database instance:
+        with shelve.open(os.fspath(path), writeback=True) as shelving:
+            yield shelving
+    
     def exports(self):
         """ Get a new dictionary instance filled with the exports. """
         out = {} # type: dict
@@ -864,6 +910,7 @@ with exporter as export:
     export(search_for_name)
     export(search_for_module)
     export(search_modules)
+    export(stringhash)
     export(determine_name)
     export(path_to_dotpath)
     export(thismodule, name='thismodule', doc="thismodule() → return the name of the module in which the `thismodule()` function was called")
@@ -948,11 +995,40 @@ def test():
         format_environment_module = search_for_module(utils.format_environment)
         assert utils == format_environment_module
     
+    @inline
+    def test_hash():
+        """ Test the string-hashing of the exporters’ dotpath """
+        expo = Exporter(path=__file__)
+        assert expo.hash() == exporter.hash()
+        print(expo.hash())
+    
+    @inline
+    def test_shelving():
+        """ Test the arbitrary data-shelving context manager """
+        from clu.fs.filesystem import TemporaryName
+        
+        with TemporaryName(randomized=True, suffix='pkl') as tempfile:
+            tempname = tempfile.name
+            with exporter.data(path=tempfile.name) as database:
+                database['yo'] = 'dogg'
+                database['i_heard'] = 'you like dicts'
+            with exporter.data(path=tempfile.name) as database:
+                assert database['yo'] == 'dogg'
+                assert database['i_heard'] == 'you like dicts'
+                assert len(database) == 2
+        
+        assert not os.path.exists(tempname)
+    
     @inline.diagnostic
     def show_search_by_id_cache_info():
         print("SEARCH-BY-ID CACHE INFO:")
         print(Exporter.cache_info())
     
+    @inline.diagnostic
+    def show_stringhash_cache_info():
+        print("STRINGHASH CACHE INFO:")
+        print(stringhash.cache_info())
+    
     return inline.test(100)
 
 if __name__ == '__main__':
