@@ -4,6 +4,7 @@ from __future__ import print_function
 import clu.abstract
 import collections.abc
 import contextlib
+import copy
 import shlex
 import sys
 
@@ -60,7 +61,7 @@ class Level(contextlib.AbstractContextManager,
     __slots__ = ('value', 'tab')
     
     def __init__(self, initial_value=0, tab=SPACETABS):
-        self.value = initial_value
+        self.value = abs(initial_value)
         self.tab = tab
     
     def indent(self, string):
@@ -69,6 +70,9 @@ class Level(contextlib.AbstractContextManager,
     def __int__(self):
         return self.value
     
+    def __index__(self):
+        return self.value
+    
     def __enter__(self):
         self.value += 1
         return self
@@ -83,6 +87,9 @@ leaf_predicate = lambda node: node.is_leafnode()
 
 @export
 class NodeBase(collections.abc.Hashable,
+               collections.abc.Collection,
+               clu.abstract.Cloneable,
+               clu.abstract.ReprWrapper,
                metaclass=clu.abstract.Slotted):
     
     """ The base class for all tree nodes. """
@@ -118,8 +125,8 @@ class NodeBase(collections.abc.Hashable,
     
     @property
     def nsname(self):
-        """ The fully namespaced name of this node. Namespaces are enumerated
-            from the root upward.
+        """ The fully namespaced name of the current node. Namespaces are
+            enumerated from the current node on upward to the root.
         """
         if self.is_rootnode():
             return self.node_name
@@ -158,6 +165,8 @@ class NodeBase(collections.abc.Hashable,
             if child in set(self.child_nodes.values()):
                 thistype = nameof(typeof(self))
                 raise ValueError(f"WTF: {thistype} “{child!s}” is already a child")
+            if child.node_parent != self:
+                child.node_parent = self
             self.child_nodes[child.name] = child
     
     def add_child(self, name, value=None):
@@ -166,7 +175,7 @@ class NodeBase(collections.abc.Hashable,
             Specify a name, and optionally a value for the node.
         """
         node = Node(parent=self, name=name, value=value)
-        self._append_nodes(node)
+        self.child_nodes[node.name] = node
         return node
     
     def has_child(self, nskey):
@@ -226,9 +235,25 @@ class NodeBase(collections.abc.Hashable,
         yield from filterfalse(leaf_predicate, self.child_nodes.values())
     
     def to_dict(self):
-        return { 'name' : self.name,
-                'value' : self.value,
-               'parent' : self.node_parent.name }
+        """ Recursively dict-ify this node and any children it may have """
+        out = { 'name' : self.name }
+        if self.value:
+            out['value'] = self.value
+        if (not self.is_leafnode()) and len(self):
+            out['children'] = tuple(child.to_dict() for child in self)
+        return out
+    
+    def clone(self, deep=False, memo=None):
+        """ Clone the current node, its leafnodes, and possibly
+            all of its children, recursively.
+        """
+        replica = type(self)(name=str(self.node_name),
+                             value=copy.copy(self.node_value),
+                             parent=True)
+        iterable = deep and self or self.leaves()
+        cloner = lambda node: node.clone(deep=deep)
+        replica._append_nodes(*map(cloner, iterable))
+        return replica
     
     def __len__(self):
         return len(self.child_nodes)
@@ -237,14 +262,19 @@ class NodeBase(collections.abc.Hashable,
         yield from self.child_nodes.values()
     
     def __getitem__(self, idx):
-        if isinstance(idx, (int, slice)):
-            return tuple(self.child_nodes.values())[idx]
-        elif isnormative(idx):
-            return self.get_child(str(idx))
-        thistype = nameof(typeof(self))
-        badtype = nameof(typeof(idx))
-        message = f"{thistype} indices must be integers, slices, or strings – not {badtype}"
-        raise TypeError(message)
+        match idx:
+            case int() | slice():
+                return tuple(self.child_nodes.values())[idx]
+            case _ if isnormative(idx):
+                return self.get_child(str(idx))
+            case _:
+                thistype = nameof(typeof(self))
+                badtype = nameof(typeof(idx))
+                message = f"{thistype} indices must be integers, slices, or strings – not {badtype}"
+                raise TypeError(message)
+    
+    def __contains__(self, nskey):
+        return self.has_child(nskey)
     
     def __hash__(self):
         # So nice it’s thrice!
@@ -258,7 +288,7 @@ class NodeBase(collections.abc.Hashable,
     def __str__(self):
         return self.name
     
-    def __repr__(self):
+    def inner_repr(self):
         child_count = len(self.child_nodes)
         out = f"Node({self.node_name})"
         if self.node_value:
@@ -286,6 +316,12 @@ class RootNode(NodeBase):
                                         value=None) # value isn’t necessary
         return instance
     
+    @classmethod
+    def populate(cls, *arguments):
+        instance = cls()
+        instance.populate_with_arguments(*arguments)
+        return instance
+    
     @property
     def value(self):
         """ A root node has no value, by definition. """
@@ -294,6 +330,19 @@ class RootNode(NodeBase):
     def is_rootnode(self):
         return True
     
+    def clone(self, deep=False, memo=None):
+        """ Clone the root node, its leafnodes, and possibly
+            all of its children, recursively.
+        """
+        replica = type(self)(name=str(self.node_name))
+        iterable = deep and self or self.leaves()
+        cloner = lambda node: node.clone(deep=deep)
+        replica._append_nodes(*map(cloner, iterable))
+        return replica
+    
+    def inner_repr(self):
+        return self.assemble_subcommand(recursive=True)
+    
     @staticmethod
     def parse_argument_to_child(argument, parent):
         """ Static method for parsing an argument into its values,
@@ -316,11 +365,11 @@ class RootNode(NodeBase):
         else:
             # It’s a namespace:
             name, value = argument, None
-    
+        
         # Add and recover a new node, containing the values
         # we parsed out:
         node = parent.add_child(name=name, value=value)
-    
+        
         # Return the node if it’s a namespace, otherwise
         # hand back the original parent:
         return argument.startswith('--') and parent or node
@@ -330,6 +379,7 @@ class RootNode(NodeBase):
         node = self
         for argument in arguments:
             node = self.parse_argument_to_child(argument, parent=node)
+        return self
 
 @export
 class Node(NodeBase):
@@ -348,9 +398,7 @@ class Node(NodeBase):
         return instance
 
 # Used in RootNode._append_nodes(…):
-acceptable_types = set(NodeBase.__mro__)
-acceptable_types.add(Node)
-acceptable_types.add(RootNode)
+acceptable_types = { NodeBase, RootNode, Node }
 
 @export
 def node_repr(node):
@@ -373,37 +421,6 @@ def tree_repr(node, level):
         with level:
             yield from tree_repr(namespace, level)
 
-@export
-def parse_argument_to_child_node(arg, parent):
-    """ Function to parse an argument into its values,
-        and then add a node with those corresponding values
-        to a provided parent node – then returning this
-        parent node (if we created a leaf) or our freshly
-        created node (if we created a namespace).
-        … this allows us to keep attaching stuff to whatever
-        gets returned here, wherever we are in the process
-        of parsing the command line.
-    """
-    # Examine the argument:
-    if arg.startswith('--'):
-        if '=' in arg:
-            # It’s a leaf with a value specified:
-            name, value = arg.removeprefix('--').split('=')
-        else:
-            # It’s a leaf with no value provided:
-            name, value = arg.removeprefix('--'), None
-    else:
-        # It’s a namespace:
-        name, value = arg, None
-    
-    # Add and recover a new node, containing the values
-    # we parsed out:
-    node = parent.add_child(name=name, value=value)
-    
-    # Return the node if it’s a namespace, otherwise
-    # hand back the original parent:
-    return arg.startswith('--') and parent or node
-
 # NodeTreeMap – a NamespaceWalker-derived KeyMap hosting a node tree
 
 @export
@@ -450,7 +467,7 @@ class NodeTreeMap(NamespaceWalker, clu.abstract.ReprWrapper,
                 except KeyError:
                     node = node.add_child(nsfragment)
         
-        # With namespaces in places, go through the items,
+        # With namespaces in place, go through the items,
         # using the newly created namespaces to anchor
         # namespaced items as needed:
         for nskey, value in interim.items():
@@ -477,16 +494,39 @@ class NodeTreeMap(NamespaceWalker, clu.abstract.ReprWrapper,
                 badtype = nameof(typeof(tree))
                 raise TypeError(f"NodeTreeMap requires a Node instance, not type {badtype}")
         self.tree = tree
-        # N.B. – deal with updates here
+        # N.B. – assume updates is a basic KeyMap:
+        if updates:
+            for nskey, value in updates.items():
+                key, namespace = unpack_ns(nskey)
+                node = self.tree
+                for nsfragment in namespace:
+                    try:
+                        node = node.namespace(nsfragment)
+                    except KeyError:
+                        node = node.add_child(nsfragment)
+                node.add_child(key, value)
     
     def walk(self):
         yield from treewalk(self.tree)
     
+    def submap(self, *namespaces, unprefixed=False):
+        if unprefixed:
+            return { node.name : self[node.name] for node in self.tree.leaves() }
+        if not namespaces:
+            return self.flatten(cls=dict)
+        node = self.tree
+        for namespace in namespaces:
+            node = node.namespace(namespace)
+        return { child.name : self[child.nsname] for child in node.leaves() }
+    
     def inner_repr(self):
         return repr(self.tree)
     
     def clone(self, deep=False, memo=None):
-        pass
+        """ Replicate the node tree map, possibly recursively """
+        replica = type(self)()
+        replica.tree = self.tree.clone(deep=deep)
+        return replica
     
     def to_dict(self):
         """ Used by `clu.config.codecs` to serialize the NodeTreeMap """
@@ -522,19 +562,19 @@ def test():
     def test_nodebase_basics():
         """ Check some of the basic NodeBase functions """
         emptynode = NodeBase(parent=None, name='yo')
-        print("EMPTY NODE: {emptynode!s}")
+        print(f"EMPTY NODE: {emptynode!s}")
         print(repr(emptynode))
         print()
         
         datanode = NodeBase(parent=None, name='dogg', value="ALL_TYPES_OF_SHIT")
-        print("DATA NODE: {datanode!s}")
+        print(f"DATA NODE: {datanode!s}")
         print(repr(datanode))
         print()
         
         emptynode.add_child('i_heard', value="you like")
         emptynode._append_nodes(datanode)
         
-        print("EMPTY NODE REDUX: {emptynode!s}")
+        print(f"EMPTY NODE REDUX: {emptynode!s}")
         print(repr(emptynode))
         print()
         
@@ -602,6 +642,36 @@ def test():
     def test_parse_command_line():
         """ Transform a command into a node tree """
         
+        def parse_argument_to_child_node(arg, parent):
+            """ Function to parse an argument into its values,
+                and then add a node with those corresponding values
+                to a provided parent node – then returning this
+                parent node (if we created a leaf) or our freshly
+                created node (if we created a namespace).
+                … this allows us to keep attaching stuff to whatever
+                gets returned here, wherever we are in the process
+                of parsing the command line.
+            """
+            # Examine the argument:
+            if arg.startswith('--'):
+                if '=' in arg:
+                    # It’s a leaf with a value specified:
+                    name, value = arg.removeprefix('--').split('=')
+                else:
+                    # It’s a leaf with no value provided:
+                    name, value = arg.removeprefix('--'), None
+            else:
+                # It’s a namespace:
+                name, value = arg, None
+    
+            # Add and recover a new node, containing the values
+            # we parsed out:
+            node = parent.add_child(name=name, value=value)
+    
+            # Return the node if it’s a namespace, otherwise
+            # hand back the original parent:
+            return arg.startswith('--') and parent or node
+                
         # Create an empty tree:
         root = RootNode()
         
@@ -699,8 +769,7 @@ def test():
         """ Check the basic functions of NodeTreeMap """
         
         # Fill a tree, per the command line:
-        root = RootNode()
-        root.populate_with_arguments(*nsflags)
+        root = RootNode.populate(*nsflags)
         
         # Stick it in a NodeTreeMap:
         itemlist = []
@@ -718,8 +787,7 @@ def test():
         """ Check node namespaced names """
         
         # Fill a tree, per the command line:
-        root = RootNode()
-        root.populate_with_arguments(*nsflags)
+        root = RootNode.populate(*nsflags)
         
         assert root.get_child('key0').nsname == 'key0'
         assert root.get_child('key0').value == 'yo'
@@ -762,8 +830,7 @@ def test():
         """ Check NodeTreeMap to/from dict functions """
         
         # Fill a tree, per the command line:
-        root = RootNode()
-        root.populate_with_arguments(*nsflags)
+        root = RootNode.populate(*nsflags)
         ntm = NodeTreeMap(tree=root)
         
         instance_dict = ntm.to_dict()
@@ -784,8 +851,7 @@ def test():
         from clu.config.codecs import json_encode, json_decode
         
         # Fill a tree, per the command line:
-        root = RootNode()
-        root.populate_with_arguments(*nsflags)
+        root = RootNode.populate(*nsflags)
         ntm = NodeTreeMap(tree=root)
         
         ntm_json = json_encode(ntm)
@@ -796,7 +862,80 @@ def test():
         print(ntm_json)
         print()
     
+    @inline
+    def test_nodetree_halfviz():
+        """ Generate Halfviz from a node tree """
+        # Fill a tree, per the command line:
+        root = RootNode.populate(*nsflags)
+        
+        def edge_repr(parent, node):
+            return f"{parent.name} -> {node.name}"
+        
+        def tree_repr(parent, node, level):
+            with level:
+                if parent:
+                    yield level.indent(edge_repr(parent, node))
+                for child in node:
+                    yield from tree_repr(node, child, level)
+        
+        for line in tree_repr(None, root, Level()):
+            print(line)
+        
+        print()
+    
+    @inline
+    def test_nodetree_clone():
+        """ Clone a node tree """
+        # Fill a tree, per the command line:
+        root = RootNode.populate(*nsflags)
+        toor = root.clone(deep=True)
+        roor = root.clone()
+        
+        # custom node_repr(…) to show node IDs:
+        def node_repr(node):
+            if not node.is_leafnode():
+                child_count = len(node)
+                return f"• {node!s} → [{child_count}] ({id(node)})"
+            if not node.value:
+                return f"• {node!s} ({hash(node)})"
+            return f"• {node!s} = {node.value} ({id(node)})"
+
+        def tree_repr(node, level):
+            yield level.indent(node_repr(node))
+            for leaf in node.leaves():
+                with level:
+                    yield level.indent(node_repr(leaf))
+            for namespace in node.namespaces():
+                with level:
+                    yield from tree_repr(namespace, level)
+        
+        for line in tree_repr(root, Level()):
+            print(line)
+        
+        print(repr(root))
+        print()
+        
+        for line in tree_repr(toor, Level()):
+            print(line)
+        
+        print(repr(toor))
+        print()
+        
+        for line in tree_repr(roor, Level()):
+            print(line)
+        
+        print(repr(roor))
+        print()
+    
+    @inline
+    def test_nodetree_to_dict():
+        """ Recursively dict-ify a node tree """
+        # Fill a tree, per the command line:
+        root = RootNode.populate(*nsflags)
+        dicts = root.to_dict()
+        pprint(dicts)
+    
     return inline.test(100)
 
 if __name__ == '__main__':
-    sys.exit(test())
+    sys.exit(test())
\ No newline at end of file
